# Importer des fonctions ou classes spécifiques pour les rendre accessibles directement
from ._mainAcoustic import *
from .AcousticEnums import *
from .AcousticTools import *
from .FocusedWave import *
from .IrregularWave import *
from .PlaneWave import *
from .StructuredWave import *


# Docstring pour documenter le package
"""
AOT_Acoustic est un package pour le traitement acoustique.
Il fournit des outils et des classes pour travailler avec des ondes acoustiques.
"""
